//------------------------------------------------------------------
// as described by M.H.Pee
// December 2004 TASC article "Trend trigger Factor"
//------------------------------------------------------------------
#property copyright "mladen"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 LightSeaGreen
#property indicator_color2 DimGray
#property indicator_color3 LimeGreen
#property indicator_color4 PaleVioletRed
#property indicator_width3 1
#property indicator_width4 1
#property indicator_level1 0.0
#property indicator_levelcolor DimGray

//
//
//
//
//

extern string TimeFrame  = "Current time frame";
extern int    TTFbars    =  15;
extern int    TopLine    =  75;
extern int    BottomLine = -75;

//
//
//
//
//

double MainBuffer[];
double SignalBufferUp[];
double SignalBufferDn[];
double SignalBuffer[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;


//------------------------------------------------------------------
//
//------------------------------------------------------------------
int init()
{
   SetIndexBuffer(0, MainBuffer);     SetIndexLabel(0,"TTF");
   SetIndexBuffer(1, SignalBuffer);   SetIndexLabel(1,"Signal");
   SetIndexBuffer(2, SignalBufferUp); SetIndexLabel(2,"SignalUp");   SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,159);
   SetIndexBuffer(3, SignalBufferDn); SetIndexLabel(3,"SignalDown"); SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,159);
      
      //
      //
      //
      //
      //
   
        indicatorFileName = WindowExpertName();
        returnBars        = (TimeFrame == "returnBars");     if (returnBars)     return(0);
        calculateValue    = (TimeFrame == "calculateValue"); if (calculateValue) return(0);
        timeFrame         = stringToTimeFrame(TimeFrame);   

     //
     //
     //
     //
     //

   IndicatorShortName(timeFrameToString(timeFrame)+" Trend Trigger Factor ("+TTFbars+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { MainBuffer[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      for(i=limit; i>=0; i--)
      {
         double HighestHighRecent= High[iHighest(NULL,0,MODE_HIGH,TTFbars,i)];
         double HighestHighOlder = High[iHighest(NULL,0,MODE_HIGH,TTFbars,i+TTFbars)];
         double LowestLowRecent  = Low[iLowest(NULL,0,MODE_LOW,TTFbars,i)];
         double LowestLowOlder   = Low[iLowest(NULL,0,MODE_LOW,TTFbars,i+TTFbars)];
         double BuyPower         = HighestHighRecent-LowestLowOlder;
         double SellPower        = HighestHighOlder -LowestLowRecent;
  
      //
      //
      //
      //
      //
      
      MainBuffer[i] = (BuyPower-SellPower)/(0.5*(BuyPower+SellPower))*100;
      if (MainBuffer[i]>=0) 
               SignalBuffer[i]=TopLine;
      else     SignalBuffer[i]=BottomLine;
      if (MainBuffer[i]>=TopLine) 
               {SignalBufferUp[i]=TopLine;SignalBufferDn[i]=EMPTY_VALUE ;}
      if (MainBuffer[i]<=BottomLine) 
               {SignalBufferDn[i]=BottomLine;SignalBufferUp[i]=EMPTY_VALUE; }
      if (MainBuffer[i]>BottomLine && MainBuffer[i]<TopLine) 
               {SignalBufferUp[i]=EMPTY_VALUE; SignalBufferDn[i]=EMPTY_VALUE;}
      }
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period())); 
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         MainBuffer[i]     = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TTFbars,TopLine,BottomLine,0,y);
         SignalBuffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TTFbars,TopLine,BottomLine,1,y);
         SignalBufferUp[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TTFbars,TopLine,BottomLine,2,y);
         SignalBufferDn[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TTFbars,TopLine,BottomLine,3,y);
   }
   return(0);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
// 
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}